module Elastify
    module Helpers
        module ElasticSearch
            class Connector

                def self.create(options, data)
                    if data.blank?
                        raise :elastify__create__required_data
                    end
                    if data[:id].blank?
                        raise :elastify__create__required_data_id
                    end
                    url = "#{options[:base_url]}/#{options[:index]}/#{options[:type]}/#{data[:id]}"
                    JSON.parse(RestClient.put(url, data.to_json, {}))
                end

                def self.update(options, data)
                    if data.blank?
                        raise :elastify__update__required_data
                    end
                    if data[:id].blank?
                        raise :elastify__update__required_data_id
                    end
                    url = "#{options[:base_url]}/#{options[:index]}/#{options[:type]}/#{data[:id]}"
                    JSON.parse(RestClient.put(url, data.to_json, {})).to_hash
                end

                def self.destroy(options, data)
                    if data.blank?
                        raise :elastify__delete__required_data
                    end
                    if data[:id].blank?
                        raise :elastify__delete__required_data_id
                    end
                    url = "#{options[:base_url]}/#{options[:index]}/#{options[:type]}/#{data[:id]}"
                    JSON.parse(RestClient.delete(url)).to_hash
                end

                def self.search(options, dsl, scroll_timeout)
                    raise :elastify__search__required_dsl unless dsl.present?
                    scroll_timeout ||= options[:scroll_timeout]
                    client = elasticsearch_client options
                    response = client.search body: dsl, index: options[:index], type: options[:type], scroll: scroll_timeout
                    Elastify::Helpers::ElasticSearch::SearchResultCollection.new(response, options)
                end

                def self.scroll(options, scroll_id, scroll_timeout)
                    raise :elastify__search__required_scroll_id unless scroll_id.present?
                    scroll_timeout ||= options[:scroll_timeout]
                    dsl = { scroll_id: scroll_id, scroll: scroll_timeout }
                    client = elasticsearch_client options
                    response = client.scroll body: dsl
                    Elastify::Helpers::ElasticSearch::SearchResultCollection.new(response, options)
                end

                def self.create_index(options)
                    url = "#{options[:base_url]}/#{options[:index]}"
                    JSON.parse(RestClient.put(url, {}.to_json, {})).to_hash
                end

                def self.destroy_index(options)
                    url = "#{options[:base_url]}/#{options[:index]}"
                    JSON.parse(RestClient.delete(url)).to_hash
                end

                def self.create_mapping(options)
                    url = "#{options[:base_url]}/#{options[:index]}/_mappings/#{options[:type]}"
                    JSON.parse(RestClient.put(url, options[:map].squish, {})).to_hash
                end

                private
                def self.elasticsearch_client options
                    Elasticsearch::Client.new host: options[:base_url]
                end
            end
        end
    end
end
